@@ -44,23 +44,23 @@ kcountb <- function(p){
 	if(sf)v <- sh1(p[nps1:np])
 	z <- .C("kcountb",
 		p=as.double(p),
-		y=as.double(response$response$times),
+		y=as.double(resp$response$times),
 		origin=as.double(origin),
-		c=as.integer(response$response$y),
-		x=as.double(response$ccov$ccov),
+		c=as.integer(resp$response$y),
+		x=as.double(resp$ccov$ccov),
 		nind=as.integer(nind),
-		nobs=as.integer(response$response$nobs),
-		nbs=as.integer(length(response$response$y)),
+		nobs=as.integer(resp$response$nobs),
+		nbs=as.integer(length(resp$response$y)),
 		nccov=as.integer(nccov),
 		model=as.integer(mdl),
 		density=as.integer(density),
 		dep=as.integer(dep),
 		birth=as.integer(birth),
 		tvc=as.integer(tvc),
-		tvcov=as.double(response$tvcov$tvcov),
+		tvcov=as.double(resp$tvcov$tvcov),
 		fit=as.integer(0),
-		pred=double(length(response$response$y)),
-		rpred=double(length(response$response$y)),
+		pred=double(length(resp$response$y)),
+		rpred=double(length(resp$response$y)),
 		rf=as.integer(rf),
 		bb=as.double(b),
 		sf=as.integer(sf),
@@ -73,20 +73,20 @@ countfb <- function(p){
 	if(sf)v <- sh1(p[nps1:np])
 	z <- .C("countfb",
 		p=as.double(p),
-		y=as.double(response$response$times),
-		c=as.integer(response$response$y),
-		x=as.double(response$ccov$ccov),
+		y=as.double(resp$response$times),
+		c=as.integer(resp$response$y),
+		x=as.double(resp$ccov$ccov),
 		nind=as.integer(nind),
-		nobs=as.integer(response$response$nobs),
-		nbs=as.integer(length(response$response$y)),
+		nobs=as.integer(resp$response$nobs),
+		nbs=as.integer(length(resp$response$y)),
 		nccov=as.integer(nccov),
 		model=as.integer(mdl),
 		density=as.integer(density),
 		tvc=as.integer(tvc),
-		tvcov=as.double(response$tvcov$tvcov),
+		tvcov=as.double(resp$tvcov$tvcov),
 		fit=as.integer(0),
-		pred=double(length(response$response$y)),
-		rpred=double(length(response$response$y)),
+		pred=double(length(resp$response$y)),
+		rpred=double(length(resp$response$y)),
 		rf=as.integer(rf),
 		bb=as.double(b),
 		sf=as.integer(sf),
@@ -103,8 +103,12 @@ tmp <- c("Markov","serial","event","cumulated","count","kalman","time")
 dep <- match(update <- match.arg(update,tmp),tmp)
 rf <- !is.null(mu)
 sf <- !is.null(shape)
-if(!inherits(response,"repeated")){
-	if(!inherits(response,"response"))response <- restovec(response,times)
+respenv <- inherits(response,"repeated")
+envname <- if(respenv)paste(deparse(substitute(response)))
+	else NULL
+if(!respenv){
+	if(!inherits(response,"response"))resp <- restovec(response,times)
+	else resp <- response
 	if(is.null(ccov))nccov <- 0
 	else {
 		if(!inherits(ccov,"tccov")){
@@ -117,7 +121,8 @@ if(!inherits(response,"repeated")){
 					ccname <- tmp}}
 			ccov <- tcctomat(ccov,names=ccname)}
 		nccov <- if(rf) 0 else ncol(ccov$ccov)}
-	if(!is.null(tvcov)){
+	if(is.null(tvcov))ttvc <- 0
+	else {
 		if(!inherits(tvcov,"tvcov")){
 			tvcname <- paste(deparse(substitute(tvcov)))
 			if(is.list(tvcov)&&ncol(tvcov[[1]])>1){
@@ -129,57 +134,77 @@ if(!inherits(response,"repeated")){
 				else tvcname <- colnames(tvcov[[1]])}
 			tvcov <- tvctomat(tvcov, names=tvcname)}
 		ttvc <- if(rf) 0 else ncol(tvcov$tvcov)}
-	else ttvc <- 0
-	response <- rmna(response=response, tvcov=tvcov, ccov=ccov)
+	resp <- rmna(response=resp, tvcov=tvcov, ccov=ccov)
 	if(!is.null(ccov))rm(ccov)
 	if(!is.null(tvcov))rm(tvcov)}
 else {
-	nccov <- if(rf||is.null(response$ccov$ccov)) 0
-		 else  ncol(response$ccov$ccov)
-	ttvc <- if(rf||is.null(response$tvcov$tvcov)) 0
-		 else  ncol(response$tvcov$tvcov)}
+	if(!rf){
+		resp <- response
+		if(is.null(ccov))resp$ccov <- NULL
+		else if(inherits(ccov,"formula"))
+			resp$ccov$ccov <- attr(finterp(ccov,envir=response,expand=F,name=paste(deparse(substitute(response)))),"model")[,-1,drop=F]
+		else stop("ccov must be a W&R formula")
+		if(is.null(tvcov))resp$tvcov <- NULL
+		else if(inherits(tvcov,"formula"))
+			resp$tvcov$tvcov <- attr(finterp(tvcov,envir=response,name=paste(deparse(substitute(response)))),"model")[,-1,drop=F]
+		else stop("tvcov must be a W&R formula")}
+	else resp <- rmna(response$response)
+	nccov <- if(rf||is.null(resp$ccov$ccov)) 0
+		 else  ncol(resp$ccov$ccov)
+	ttvc <- if(rf||is.null(resp$tvcov$tvcov)) 0
+		 else  ncol(resp$tvcov$tvcov)}
 if((inherits(envir,"repeated")&&
-	(length(response$response$nobs)!=length(envir$response$nobs)||
-	any(response$response$nobs!=envir$response$nobs)))||
+	(length(resp$response$nobs)!=length(envir$response$nobs)||
+	any(resp$response$nobs!=envir$response$nobs)))||
 	(inherits(envir,"tvcov")&&
-	(length(response$response$nobs)!=length(envir$tvcov$nobs)||
-	any(response$response$nobs!=envir$tvcov$nobs))))
+	(length(resp$response$nobs)!=length(envir$tvcov$nobs)||
+	any(resp$response$nobs!=envir$tvcov$nobs))))
 	stop("response and envir objects are incompatible")
-if(is.null(response$response$times))stop("These models cannot be fitted without times.")
-mu3 <- sh3 <- name <- NULL
-if(inherits(envir,"repeated")||inherits(envir,"tccov")){
-	type <- if(inherits(envir,"repeated"))"repeated"
+if(is.null(resp$response$times))stop("These models cannot be fitted without times.")
+mu3 <- sh3 <- NULL
+if(respenv||inherits(envir,"repeated")||inherits(envir,"tccov")){
+	type <- if(respenv||inherits(envir,"repeated"))"repeated"
 		else if(inherits(envir,"tccov"))"tccov"
 		else "tvcov"
-	name <- paste(deparse(substitute(envir)))
+	if(is.null(envname))envname <- paste(deparse(substitute(envir)))
 	if(inherits(mu,"formula")){
-		mu3 <- finterp(mu)
+		mu3 <- if(respenv)finterp(mu,envir=response,name=envname)
+			else finterp(mu,envir=envir,name=envname)
 		class(mu) <- c(class(mu),type)}
 	else if(is.function(mu)){
-		mu3 <- mu
-		attributes(mu3) <- attributes(fnenvir(mu))
+		tmp <- parse(text=paste(deparse(mu))[-1])
 		class(mu) <- type
-		mu <- fnenvir(mu,envir=envir,name=name)}
+		mu <- if(respenv)fnenvir(mu,envir=response,name=envname)
+			else fnenvir(mu,envir=envir,name=envname)
+		mu3 <- mu
+		if(respenv)attr(mu3,"model") <- tmp}
 	if(inherits(shape,"formula")){
-		sh3 <- finterp(shape)
+		sh3 <- if(respenv)finterp(shape,envir=response,name=envname)
+			else finterp(shape,envir=envir,name=envname)
 		class(shape) <- c(class(shape),type)}
 	else if(is.function(shape)){
-		sh3 <- shape
-		attributes(sh3) <- attributes(fnenvir(shape))
+		tmp <- parse(text=paste(deparse(shape))[-1])
 		class(shape) <- type
-		shape <- fnenvir(shape,envir=envir,name=name)}}
+		shape <- if(respenv)fnenvir(shape,envir=response,name=envname)
+			else fnenvir(shape,envir=envir,name=envname)
+		sh3 <- shape
+		if(respenv)attr(sh3,"model") <- tmp}}
 npreg <- length(preg)
 mu1 <- sh1 <- v <- b <- NULL
 if(inherits(mu,"formula")){
 	pr <- if(npreg>0)preg else ptvc
 	npr <- length(pr)
-	mu2 <- finterp(mu,envir=envir,name=name,expand=is.null(preg))
+	mu2 <- if(respenv)finterp(mu,envir=response,name=envname,expand=is.null(preg))
+		else finterp(mu,envir=envir,name=envname,expand=is.null(preg))
 	npt1 <- length(attr(mu2,"parameters"))
 	if(is.matrix(attr(mu2,"model"))){
 		if(all(dim(attr(mu2,"model"))==1)){
-			mu1 <- function(p) p[1]*rep(1,n)
+			mu1 <- function(p) exp(p[1]*rep(1,n))
 			attributes(mu1) <- attributes(mu2)
-			mu2 <- NULL}}
+			mu2 <- NULL}
+		else {
+			mu1 <- function(p) exp(mu2(p))
+			attributes(mu1) <- attributes(mu2)}}
 	else {
 		if(npr!=npt1&&length(ptvc)!=npt1){
 			cat("\nParameters are ")
@@ -197,15 +222,19 @@ if(inherits(mu,"formula")){
 			cv <- covind(response)
 			mu1 <- function(p) mu2(p)[cv]
 			attributes(mu1) <- attributes(mu2)}
-		else {
+		else if(!is.matrix(attr(mu2,"model"))){
 			mu1 <- mu2
 			rm(mu2)}}}
 else if(is.function(mu))mu1 <- mu
 if(!is.null(mu1)&&is.null(attributes(mu1))){
 	attributes(mu1) <- if(is.function(mu)){
-		if(!inherits(mu,"formulafn"))attributes(fnenvir(mu))
+		if(!inherits(mu,"formulafn")){
+			if(respenv)attributes(fnenvir(mu,envir=response))
+			else attributes(fnenvir(mu,envir=envir))}
 		else attributes(mu)}
-		else attributes(fnenvir(mu1))}
+		else {
+			if(respenv)attributes(fnenvir(mu1,envir=response))
+			else attributes(fnenvir(mu1,envir=envir))}}
 nlp <- if(is.function(mu1))length(attr(mu1,"parameters"))
 	else if(is.null(mu1))NULL
 	else npt1
@@ -216,7 +245,8 @@ if(!is.null(nlp)){
 		stop(paste("ptvc should have",nlp,"initial estimates"))}
 nps <- length(pshape)
 if(inherits(shape,"formula")){
-	sh2 <- finterp(shape,envir=envir,name=name)
+	sh2 <- if(respenv)finterp(shape,envir=response,name=envname)
+		else finterp(shape,envir=envir,name=envname)
 	npt2 <- length(attr(sh2,"parameters"))
 	if(is.matrix(attr(sh2,"model"))){
 		if(all(dim(attr(sh2,"model"))==1)){
@@ -245,9 +275,13 @@ if(inherits(shape,"formula")){
 else if(is.function(shape))sh1 <- shape
 if(!is.null(sh1)&&is.null(attributes(sh1)))
 	attributes(sh1) <- if(is.function(shape)){
-		if(!inherits(shape,"formulafn"))attributes(fnenvir(shape))
+		if(!inherits(shape,"formulafn")){
+			if(respenv)attributes(fnenvir(shape,envir=response))
+			else attributes(fnenvir(shape,envir=envir))}
 		else attributes(shape)}
-		else attributes(fnenvir(sh1))
+		else {
+			if(respenv)attributes(fnenvir(sh1,envir=response))
+			else attributes(fnenvir(sh1,envir=envir))}
 nlp <- if(is.function(shape))length(attr(sh1,"parameters"))
 	else if(is.null(shape))NULL
 	else npt2
@@ -292,18 +326,18 @@ else if(depend=="frailty"){
 if(rf&&npreg>0)nccov <- npreg-1
 if(!rf&&1+nccov!=npreg)
 	stop(paste(1+nccov,"regression estimates must be supplied"))
-nind <- length(response$response$nobs)
-if(ttvc>0&&tvc!=ttvc)stop(paste(ttvc,"initial estimates of coefficients for time-varying covariates must be supplied"))
+nind <- length(resp$response$nobs)
+if(!rf&&(ttvc>0&&tvc!=ttvc||ttvc==0&&tvc>0))stop(paste(ttvc,"initial estimates of coefficients for time-varying covariates must be supplied"))
 if(rf){
 	if(tvc>0&&nccov>0)stop("With a mean function, initial estimates must be supplied either in preg or in ptvc")
 	if(tvc>0){
-		if(length(mu1(ptvc))!=length(response$response$y))stop("The mu function or formula must provide an estimate for each observation")
+		if(length(mu1(ptvc))!=length(resp$response$y))stop("The mu function or formula must provide an estimate for each observation")
 		tvc <- tvc-1}
 	else if(length(mu1(preg))==1){
-		if(nccov==0)mu1 <- function(p) rep(p[1],length(response$response$y))
+		if(nccov==0)mu1 <- function(p) rep(p[1],length(resp$response$y))
 		else stop("Number of estimates does not correspond to mu function")}
 	else if(length(mu1(preg))!=nind)stop("The mu function or formula must provide an estimate for each individual")}
-if(sf&&length(sh1(pshape))!=length(response$response$y))stop("The shape function must provide an estimate for each observation")
+if(sf&&length(sh1(pshape))!=length(resp$response$y))stop("The shape function must provide an estimate for each observation")
 np <- 1+nccov+tvc+1+(depend=="serial")+birth+nps+(!is.null(pintercept))
 nps1 <- np-nps-(!is.null(pintercept))+1
 p <- c(preg,pbirth,ptvc,pinitial,pdepend,pshape,pintercept)
@@ -327,20 +361,20 @@ z <- if(depend=="frailty"){
 	if(sf)v <- sh1(z0$estimate[nps1:np])
 	.C("countfb",
         	p=as.double(z0$estimate),
-        	y=as.double(response$response$times),
-        	c=as.integer(response$response$y),
-        	x=as.double(response$ccov$ccov),
+        	y=as.double(resp$response$times),
+        	c=as.integer(resp$response$y),
+        	x=as.double(resp$ccov$ccov),
         	nind=as.integer(nind),
-        	nobs=as.integer(response$response$nobs),
-        	nbs=as.integer(length(response$response$y)),
+        	nobs=as.integer(resp$response$nobs),
+        	nbs=as.integer(length(resp$response$y)),
         	nccov=as.integer(nccov),
         	model=as.integer(mdl),
         	density=as.integer(density),
         	tvc=as.integer(tvc),
-        	tvcov=response$tvcov$tvcov,
+        	tvcov=resp$tvcov$tvcov,
         	fit=as.integer(1),
-        	pred=double(length(response$response$y)),
-		rpred=double(length(response$response$y)),
+        	pred=double(length(resp$response$y)),
+		rpred=double(length(resp$response$y)),
         	rf=as.integer(rf),
 		bb=as.double(b),
 		sf=as.integer(sf),
@@ -352,23 +386,23 @@ else {
 	if(sf)v <- sh1(z0$estimate[nps1:np])
 	.C("kcountb",
         	p=as.double(z0$estimate),
-        	y=as.double(response$response$times),
+        	y=as.double(resp$response$times),
         	origin=as.double(origin),
-        	c=as.integer(response$response$y),
-        	x=as.double(response$ccov$ccov),
+        	c=as.integer(resp$response$y),
+        	x=as.double(resp$ccov$ccov),
         	nind=as.integer(nind),
-        	nobs=as.integer(response$response$nobs),
-        	nbs=as.integer(length(response$response$y)),
+        	nobs=as.integer(resp$response$nobs),
+        	nbs=as.integer(length(resp$response$y)),
         	nccov=as.integer(nccov),
         	model=as.integer(mdl),
         	density=as.integer(density),
         	dep=as.integer(dep),
         	birth=as.integer(birth),
         	tvc=as.integer(tvc),
-        	tvcov=response$tvcov$tvcov,
+        	tvcov=resp$tvcov$tvcov,
         	fit=as.integer(1),
-        	pred=double(length(response$response$y)),
-		rpred=double(length(response$response$y)),
+        	pred=double(length(resp$response$y)),
+		rpred=double(length(resp$response$y)),
 		rf=as.integer(rf),
 		bb=as.double(b),
 		sf=as.integer(sf),
@@ -389,14 +423,14 @@ z <- list(
 	depend=depend,
 	update=update,
 	birth=birth,
-	response=response$response,
+	response=resp$response,
 	pred=z$pred,
 	rpred=z$rpred,
-	ccov=response$ccov,
-	tvcov=response$tvcov,
+	ccov=resp$ccov,
+	tvcov=resp$tvcov,
 	maxlike=z0$minimum,
 	aic=z0$minimum+np,
-	df=length(response$response$y)-np,
+	df=length(resp$response$y)-np,
 	npt=np,
 	coefficients=z0$estimate,
 	se=se,
